/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <json-glib/json-glib.h>
#include <app_debug.h>
#include <Eina.h>
#include <stdbool.h>
#include <Evas.h>
#include <viewmgr.h>
#include <gum/gum-user.h>

#include "data_home.h"
#include "datamgr.h"
#include "utils.h"
#include "view.h"
#include "defs.h"

#define STR_SELECT_ACTION_PUSH "push"
#define STR_SELECT_ACTION_LAUNCH "launch"

#define OBJECT_MEMBERS "item"
#define MEMBER_STR_NAME "name"
#define MEMBER_STR_ICON "icon"
#define MEMBER_STR_FOCUS_ICON "focus_icon"
#define MEMBER_STR_SELECT_ACTION "select_action"
#define MEMBER_STR_PARAMETER "parameter"

#define MAX_BUF 64

static inline char *_read_string(JsonReader *reader, char *member)
{
	char *val;

	if (!reader || !member) {
		_ERR("Invalid argument");
		return NULL;
	}

	json_reader_read_member(reader, member);
	val = (char *)json_reader_get_string_value(reader);
	json_reader_end_member(reader);

	return val;
}

static inline gboolean _read_boolean(JsonReader *reader, char *member)
{
	gboolean val;

	if (!reader || !member) {
		_ERR("Invalid argument");
		return false;
	}

	json_reader_read_member(reader, member);
	val = json_reader_get_boolean_value(reader);
	json_reader_end_member(reader);

	return val;
}

static bool _get_login_user(struct datamgr_item *di)
{
	GumUser *user;
	uid_t uid = getuid();
	char *name, *icon, *focus_icon;

	user = gum_user_get_sync(uid, FALSE);
	if (!user) {
		_ERR("failed to get user service");
		return false;
	}

	g_object_get(G_OBJECT(user), GUM_ATTR_NAME, &name, GUM_ATTR_ICON, &icon,
			NULL);
	if (!name || !icon) {
		_ERR("failed to get user info");
		free(name);
		free(icon);
		return false;
	}

	di->title = name;
	if (strlen(icon) == 0) {
		free(icon);
		di->icon = strdup(IMAGE_USER_DEFAULT);
		di->focus_icon = strdup(IMAGE_USER_DEFAULT_FOCUS);
	} else {
		di->icon = icon;
		focus_icon = (char *)utils_get_focus_icon_from_icon(icon);
		di->focus_icon = strdup(focus_icon);
	}

	g_object_unref(user);

	return true;
}

static struct datamgr_item *_new_datamgr_item(char *name, char *icon,
		char *focus_icon, char *parameter, char *action)
{
	struct datamgr_item *di;

	di = calloc(1, sizeof(*di));
	if (!di) {
		_ERR("failed to calloc home item");
		return NULL;
	}

	di->parameter = strdup(parameter);

	if (!strcmp(action, STR_SELECT_ACTION_PUSH) && _get_login_user(di)) {
		di->action = ITEM_SELECT_ACTION_PUSH;
		return di;
	} else if (!strcmp(action, STR_SELECT_ACTION_LAUNCH)) {
		di->action = ITEM_SELECT_ACTION_LAUNCH;
	} else {
		free(di->parameter);
		free(di);
		return NULL;
	}

	di->title = strdup(name);
	di->icon = strdup(icon);
	di->focus_icon = strdup(focus_icon);

	return di;
}

static struct datamgr_item *_pack_home_item(JsonReader *reader, int i)
{
	char *name, *icon, *parameter, *focus_icon, *action;

	if (!reader) {
		_ERR("Invalid argument");
		return NULL;
	}

	if (!json_reader_read_element(reader, i)) {
		_ERR("failed to reader %dth element", i);
		return NULL;
	}

	name = _read_string(reader, MEMBER_STR_NAME);
	if (!name)
		goto err;

	icon = _read_string(reader, MEMBER_STR_ICON);
	if (!icon)
		goto err;

	focus_icon = _read_string(reader, MEMBER_STR_FOCUS_ICON);
	if (!focus_icon)
		goto err;

	action = _read_string(reader, MEMBER_STR_SELECT_ACTION);
	parameter = _read_string(reader, MEMBER_STR_PARAMETER);

	json_reader_end_element(reader);

	return _new_datamgr_item(name, icon, focus_icon, parameter, action);
err:
	json_reader_end_element(reader);
	return NULL;
}

static Eina_List *_load_home_data(JsonReader *reader)
{
	struct datamgr_item *di;
	int cnt, i;
	Eina_List *list;

	if (!reader) {
		_ERR("Invalid argument");
		return NULL;
	}

	json_reader_read_member(reader, OBJECT_MEMBERS);
	if (!json_reader_is_array(reader)) {
		_ERR(OBJECT_MEMBERS" is not array");
		json_reader_end_member(reader);
		return NULL;
	}

	list = NULL;
	cnt = json_reader_count_elements(reader);
	for (i = 0; i < cnt; i++) {
		di = _pack_home_item(reader, i);
		if (!di)
			continue;

		list = eina_list_append(list, di);
	}

	json_reader_end_member(reader);

	return list;
}

static bool _load_home(struct datamgr *dm)
{
	JsonParser *parser;
	JsonReader *reader;

	parser = json_parser_new();
	if (!parser) {
		_ERR("failed to new parser");
		return false;
	}

	if (!json_parser_load_from_file(parser, DEFCONFIG, NULL)) {
		_ERR("failed to load from file");
		g_object_unref(parser);
		return false;
	}

	reader = json_reader_new(json_parser_get_root(parser));
	if (!reader) {
		_ERR("failed to new reader");
		g_object_unref(parser);
		return false;
	}

	dm->list = _load_home_data(reader);
	if (!dm->list) {
		_ERR("failed to load home data");
		g_object_unref(reader);
		g_object_unref(parser);
		return false;
	}

	g_object_unref(reader);
	g_object_unref(parser);

	return true;
}

static void _unload_home(struct datamgr *dm)
{
	struct datamgr_item *di;

	EINA_LIST_FREE(dm->list, di) {
		free(di->title);
		free(di->icon);
		free(di->focus_icon);
		free(di->parameter);

		free(di);
	}

	dm->list = NULL;
}

static Eina_List *_get_items(struct datamgr *dm)
{
	if (!dm) {
		_ERR("Invalid argument");
		return NULL;
	}

	_unload_home(dm);
	_load_home(dm);

	return dm->list;
}

static void _fini(struct datamgr *dm)
{
	if (!dm) {
		_ERR("Invalid argument");
		return;
	}

	_unload_home(dm);
}

static bool _init(struct datamgr *dm)
{
	if (!dm) {
		_ERR("Invalid argument");
		return false;
	}

	return _load_home(dm);
}

static int _select(struct datamgr_item *di)
{
	int r;

	if (!di || !di->parameter)
		return ITEM_SELECT_ERROR_INVALID_PARAMETER;

	r = ITEM_SELECT_ERROR_INVALID_OPERATION;
	switch (di->action) {
	case ITEM_SELECT_ACTION_LAUNCH:
		if (utils_launch_app(di->parameter, NULL))
			r = ITEM_SELECT_ERROR_NONE;
		break;
	case ITEM_SELECT_ACTION_PUSH:
		if (viewmgr_push_view(di->parameter))
			r = ITEM_SELECT_ERROR_NONE;
		break;
	default:
		_ERR("Invalid state");
		break;
	}

	return r;
}

static struct data_class dclass = {
	.init = _init,
	.fini = _fini,
	.get_items = _get_items,
	.select = _select
};

struct data_class *datamgr_home_get_dclass(void)
{
	return &dclass;
}

