/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>
#include <app_define.h>

#include "defs.h"
#include "view.h"
#include "utils.h"

#define TITLE_ACTION_MENU_RESET "Reset"
#define TITLE_ACTION_MENU_DELETE "Delete"
#define COUNT_MENU_ITEM 4
#define MAX_BUF 128

enum input_handler_type {
	INPUT_HANDLER_TYPE_BTN,
	INPUT_HANDLER_TYPE_MENU_RESET,
	INPUT_HANDLER_TYPE_MENU_DELETE,
	INPUT_HANDLER_TYPE_MENU_EMPTY,
	INPUT_HANDLER_TYPE_POPUP_RESET,
	INPUT_HANDLER_TYPE_POPUP_DISCARD,
	INPUT_HANDLER_TYPE_POPUP_DELETE,
	INPUT_HANDLER_TYPE_POPUP_CANCEL
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *btn;
	Evas_Object *popup;
	Evas_Object *tbl;
	Eina_List *list;

	bool edited;

	enum menu_type mtype;
	struct datamgr *dm;
	struct datamgr_item *di;
};

struct _menu_info {
	const char *style;
	const char *title;
	int id;
	Eina_Bool disable;
};

static struct _menu_info add_menu_info[] = {
	{
		.style = STYLE_BTN_ACTION_MENU_RESET,
		.title = TITLE_ACTION_MENU_RESET,
		.id = INPUT_HANDLER_TYPE_MENU_RESET,
		.disable = EINA_FALSE
	},
	{
		.style = STYLE_BTN_ACTION_MENU_EMPTY,
		.title = NULL,
		.id = INPUT_HANDLER_TYPE_MENU_EMPTY,
		.disable = EINA_TRUE

	},
	{
		.style = STYLE_BTN_ACTION_MENU_EMPTY,
		.title = NULL,
		.id = INPUT_HANDLER_TYPE_MENU_EMPTY,
		.disable = EINA_TRUE
	},
	{
		.style = STYLE_BTN_ACTION_MENU_EMPTY,
		.title = NULL,
		.id = INPUT_HANDLER_TYPE_MENU_EMPTY,
		.disable = EINA_TRUE
	}
};

static struct _menu_info edit_menu_info[] = {
	{
		.style = STYLE_BTN_ACTION_MENU_RESET,
		.title = TITLE_ACTION_MENU_RESET,
		.id = INPUT_HANDLER_TYPE_MENU_RESET,
		.disable = EINA_FALSE
	},
	{
		.style = STYLE_BTN_ACTION_MENU_DELETE,
		.title = TITLE_ACTION_MENU_DELETE,
		.id = INPUT_HANDLER_TYPE_MENU_DELETE,
		.disable = EINA_FALSE

	},
	{
		.style = STYLE_BTN_ACTION_MENU_EMPTY,
		.title = NULL,
		.id = INPUT_HANDLER_TYPE_MENU_EMPTY,
		.disable = EINA_TRUE
	},
	{
		.style = STYLE_BTN_ACTION_MENU_EMPTY,
		.title = NULL,
		.id = INPUT_HANDLER_TYPE_MENU_EMPTY,
		.disable = EINA_TRUE
	}
};

static void _focused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	elm_object_signal_emit(obj, SIG_FOCUS, SRC_PROG);
}

static void _unfocused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	elm_object_signal_emit(obj, SIG_UNFOCUS, SRC_PROG);
}

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev);

static void _mouse_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Down *ev);

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static input_handler handler = {
	.focused = _focused,
	.unfocused = _unfocused,
	.key_down = _key_down,
	.mouse_down = _mouse_down,
	.mouse_move = _mouse_move
};

static void _add_popup(struct _priv *priv, char *title, char *message,
		char *first_btn_message, int first_btn_id,
		char *second_btn_message, int second_btn_id)
{
	Evas_Object *popup, *first_btn, *second_btn;

	popup = utils_add_popup(priv->base, title, message);
	if (!popup) {
		_ERR("failed to add popup");
		return;
	}

	first_btn = utils_add_button(popup, first_btn_message,
			PART_POPUP_BUTTON_1);
	if (!first_btn) {
		_ERR("failed to add delete btn");
		evas_object_del(popup);
		return;
	}
	inputmgr_add_callback(first_btn, first_btn_id, &handler, priv);

	second_btn = utils_add_button(popup, second_btn_message,
			PART_POPUP_BUTTON_2);
	if (!second_btn) {
		_ERR("failed to add cancel btn");
		evas_object_del(popup);
		return;
	}
	inputmgr_add_callback(second_btn, second_btn_id, &handler, priv);

	elm_object_focus_set(second_btn, EINA_TRUE);

	priv->popup = popup;
}

static void _delete_popup(struct _priv *priv)
{
	if (!priv->popup)
		return;

	evas_object_del(priv->popup);
	priv->popup = NULL;
}

static void _delete_user(struct _priv *priv, Evas_Object *obj)
{
	char buf[MAX_BUF];

	if (!priv->di)
		return;

	priv->di->action = ITEM_SELECT_ACTION_DELETE;
	if (datamgr_select_item(priv->dm, priv->di) != ITEM_SELECT_ERROR_NONE) {
		_ERR("failed to delete user");
		return;
	}

	_delete_popup(priv);
	snprintf(buf, sizeof(buf), "%s %s", priv->di->title,
			MESSAGE_USER_DELETE);
	utils_add_toast(priv->win, buf);

	viewmgr_hide_view(VIEW_ACTION_MENU);
	viewmgr_hide_view(VIEW_USER_EDIT);
	viewmgr_show_view(VIEW_USER);
}

static void _select(int id, void *data, Evas_Object *obj)
{
	struct _priv *priv;

	priv = data;

	switch (id) {
	case INPUT_HANDLER_TYPE_BTN:
		if (priv->edited) {
			_add_popup(priv, MESSAGE_DISCARD,
					MESSAGE_DISCARD_CONTENTS,
					MESSAGE_DISCARD,
					INPUT_HANDLER_TYPE_POPUP_DISCARD,
					MESSAGE_CANCEL,
					INPUT_HANDLER_TYPE_POPUP_CANCEL);
			return;
		}
		viewmgr_update_view(VIEW_USER_EDIT, UPDATE_TYPE_RESET, NULL);
		viewmgr_hide_view(VIEW_ACTION_MENU);
		viewmgr_hide_view(VIEW_USER_EDIT);

		utils_launch_app(APP_ID_LIVETV, NULL);
		break;
	case INPUT_HANDLER_TYPE_MENU_RESET:
		_add_popup(priv, MESSAGE_RESET, MESSAGE_RESET_CONTENTS,
				MESSAGE_RESET, INPUT_HANDLER_TYPE_POPUP_RESET,
				MESSAGE_CANCEL,
				INPUT_HANDLER_TYPE_POPUP_CANCEL);
		break;
	case INPUT_HANDLER_TYPE_MENU_DELETE:
		_add_popup(priv, MESSAGE_DELETE_USER, MESSAGE_DELETE_CONTENTS,
				MESSAGE_DELETE,
				INPUT_HANDLER_TYPE_POPUP_DELETE,
				MESSAGE_CANCEL,
				INPUT_HANDLER_TYPE_POPUP_CANCEL);
		break;
	case INPUT_HANDLER_TYPE_POPUP_DISCARD:
		viewmgr_update_view(VIEW_USER_EDIT, UPDATE_TYPE_RESET, NULL);
		viewmgr_hide_view(VIEW_ACTION_MENU);
		viewmgr_hide_view(VIEW_USER_EDIT);
		_delete_popup(priv);

		utils_launch_app(APP_ID_LIVETV, NULL);
		break;
	case INPUT_HANDLER_TYPE_POPUP_RESET:
		viewmgr_update_view(VIEW_USER_EDIT, UPDATE_TYPE_RESET, NULL);
		elm_object_signal_emit(priv->base, SIG_HIDE, SRC_PROG);
		_delete_popup(priv);
		break;
	case INPUT_HANDLER_TYPE_POPUP_DELETE:
		_delete_user(priv, obj);
		break;
	case INPUT_HANDLER_TYPE_POPUP_CANCEL:
		_delete_popup(priv);
		break;
	default:
		return;
	}
}

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;
	if (!strcmp(ev->keyname, KEY_BACK) ||
			!strcmp(ev->keyname, KEY_ESC)) {
		switch (id) {
		case INPUT_HANDLER_TYPE_BTN:
		case INPUT_HANDLER_TYPE_MENU_RESET:
		case INPUT_HANDLER_TYPE_MENU_DELETE:
			elm_object_signal_emit(priv->base, SIG_HIDE, SRC_PROG);
			break;
		case INPUT_HANDLER_TYPE_POPUP_RESET:
		case INPUT_HANDLER_TYPE_POPUP_DISCARD:
		case INPUT_HANDLER_TYPE_POPUP_DELETE:
		case INPUT_HANDLER_TYPE_POPUP_CANCEL:
			_delete_popup(priv);
			break;
		default:
			return;
		}
	} else if (!strcmp(ev->keyname, KEY_ENTER)) {
		_select(id, data, obj);
		ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
	}
}

static void _mouse_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Down *ev)
{
	if (!data || !obj) {
		_ERR("Invalid argument");
		return;
	}

	_select(id, data, obj);
	ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
}

static bool _add_base(struct _priv *priv)
{
	Evas_Object *btn, *tbl;

	btn = utils_add_layout(priv->base, GRP_ACTION_MENU_LIVETV, true,
			PART_ACTION_MENU_LIVETV);
	if (!btn) {
		_ERR("failed to add btn");
		return false;
	}
	elm_object_focus_next_object_set(btn, btn, ELM_FOCUS_LEFT);
	inputmgr_add_callback(btn, INPUT_HANDLER_TYPE_BTN, &handler, priv);
	priv->btn = btn;

	tbl = utils_add_table(priv->base, true, PART_ACTION_MENU_TABLE);
	if (!tbl) {
		_ERR("failed to add table");
		return false;
	}
	elm_table_padding_set(tbl, elm_config_scale_get() * 1, 0);
	priv->tbl = tbl;

	return true;
}

static void _hide_done(void *data, Evas_Object *obj, const char *emission,
		const char *source)
{
	viewmgr_hide_view(VIEW_ACTION_MENU);
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	Evas_Object *base;

	if (!win || !data) {
		_ERR("Invalid argument");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to calloc priv");
		return NULL;
	}

	base = utils_add_layout(win, GRP_ACTION_MENU, false, NULL);
	if (!base) {
		_ERR("failed to create base");
		free(priv);
		return NULL;
	}
	evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;
	priv->dm = data;

	if (!_add_base(priv)) {
		_ERR("failed to add menu");
		free(priv);
		evas_object_del(base);
		return NULL;
	}

	viewmgr_set_view_data(VIEW_ACTION_MENU, priv);
	elm_object_signal_callback_add(base, SIG_HIDE_DONE, SRC_EDJE,
			_hide_done, NULL);

	return base;
}

static void _update_focus_sequence(struct _priv *priv)
{
	Eina_List *l;
	Evas_Object *item, *eo;

	eo = NULL;
	EINA_LIST_FOREACH(priv->list, l, item) {
		if (!elm_object_disabled_get(item)) {
			eo = item;
			break;
		}
	}

	if (eo) {
		elm_object_focus_next_object_set(priv->btn, eo, ELM_FOCUS_DOWN);
		elm_object_focus_next_object_set(eo, eo, ELM_FOCUS_LEFT);
		elm_object_focus_set(eo, EINA_TRUE);
	} else {
		elm_object_focus_next_object_set(priv->btn, priv->btn,
				ELM_FOCUS_DOWN);
		elm_object_focus_set(priv->btn, EINA_TRUE);
	}
}

static bool _add_menu_item(struct _priv *priv)
{
	Evas_Object *item;
	int i;
	struct _menu_info *menu_info;

	switch (priv->mtype) {
	case MENU_TYPE_ADD:
		menu_info = add_menu_info;
		break;
	case MENU_TYPE_EDIT:
		menu_info = edit_menu_info;
		break;
	default:
		_ERR("Invalid type");
		return false;
	}

	for (i = 0; i < COUNT_MENU_ITEM; i++) {
		item = elm_button_add(priv->tbl);
		if (!item) {
			_ERR("failed to add item");
			return false;
		}
		elm_object_style_set(item, menu_info[i].style);
		elm_object_text_set(item, menu_info[i].title);
		inputmgr_add_callback(item, menu_info[i].id, &handler,
				priv);
		evas_object_show(item);
		elm_table_pack(priv->tbl, item, i, 0, 1, 1);

		if ((i == 0 && !priv->edited) || menu_info[i].disable)
			elm_object_disabled_set(item, EINA_TRUE);

		elm_object_focus_next_object_set(item, item, ELM_FOCUS_DOWN);

		priv->list = eina_list_append(priv->list, item);
	}

	_update_focus_sequence(priv);

	return true;
}

static void _show(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	if (!_add_menu_item(priv)) {
		_ERR("failed to add item");
		return;
	}

	evas_object_show(priv->base);
	elm_object_signal_emit(priv->base, SIG_SHOW, SRC_PROG);
}

static void _hide(void *data)
{
	struct _priv *priv;
	Evas_Object *eo;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	EINA_LIST_FREE(priv->list, eo) {
		evas_object_del(eo);
		inputmgr_remove_callback(eo, &handler);
	}

	_delete_popup(priv);
	elm_table_clear(priv->tbl, EINA_TRUE);
	evas_object_hide(priv->base);
	priv->list = NULL;
	priv->edited = false;
	priv->di = NULL;
	priv->mtype = MENU_TYPE_MAX;
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	inputmgr_remove_callback(priv->btn, &handler);

	evas_object_del(priv->base);
	free(priv);
}

static void _update(void *view_data, int update_type, void *data)
{
	struct _priv *priv;
	struct view_update_data *vdata;

	if (!view_data) {
		_ERR("Invalid argument");
		return;
	}

	priv = view_data;

	switch (update_type) {
	case UPDATE_TYPE_MENU:
		if (!data)
			return;

		vdata = data;
		priv->edited = vdata->state;
		priv->mtype = vdata->type;
		if (vdata->di)
			priv->di = vdata->di;
		break;
	default:
		_ERR("Invalid type");
		return;
	}
}

static view_class vclass = {
	.view_id = VIEW_ACTION_MENU,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
	.update = _update
};

view_class *view_action_menu_get_vclass(void)
{
	return &vclass;
}
